@@ -52,7 +52,6 @@
 
 #include "nm-openvpn-service.h"
 #include "nm-utils.h"
-#include "common/utils.h"
 
 #define NM_OPENVPN_HELPER_PATH		LIBEXECDIR"/nm-openvpn-service-openvpn-helper"
 
@@ -89,9 +88,6 @@ static ValidProperty valid_properties[] = {
 	{ NM_OPENVPN_KEY_CERT,                 G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CIPHER,               G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_COMP_LZO,             G_TYPE_BOOLEAN, 0, 0, FALSE },
-	{ NM_OPENVPN_KEY_MSSFIX,               G_TYPE_BOOLEAN, 0, 0, FALSE },
-	{ NM_OPENVPN_KEY_TUNNEL_MTU,           G_TYPE_INT, 0, G_MAXINT, FALSE },
-	{ NM_OPENVPN_KEY_FRAGMENT_SIZE,        G_TYPE_INT, 0, G_MAXINT, FALSE },
 	{ NM_OPENVPN_KEY_CONNECTION_TYPE,      G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TAP_DEV,              G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_KEY,                  G_TYPE_STRING, 0, 0, FALSE },
@@ -557,12 +553,7 @@ validate_auth (const char *auth)
 	if (auth) {
 		if (   !strcmp (auth, NM_OPENVPN_AUTH_NONE)
 		    || !strcmp (auth, NM_OPENVPN_AUTH_MD5)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA1)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA224)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA256)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA384)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA512)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_RIPEMD160))
+		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA1))
 			return TRUE;
 	}
 	return FALSE;
@@ -636,43 +627,6 @@ add_openvpn_arg_int (GPtrArray *args, const char *arg)
 	return TRUE;
 }
 
-static void
-add_cert_args (GPtrArray *args, NMSettingVPN *s_vpn)
-{
-	const char *ca, *cert, *key;
-
-	g_return_if_fail (args != NULL);
-	g_return_if_fail (s_vpn != NULL);
-
-	ca = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
-	cert = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
-	key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-
-	if (   ca && strlen (ca)
-	    && cert && strlen (cert)
-	    && key && strlen (key)
-	    && !strcmp (ca, cert)
-	    && !strcmp (ca, key)) {
-		add_openvpn_arg (args, "--pkcs12");
-		add_openvpn_arg (args, ca);
-	} else {
-		if (ca && strlen (ca)) {
-			add_openvpn_arg (args, "--ca");
-			add_openvpn_arg (args, ca);
-		}
-
-		if (cert && strlen (cert)) {
-			add_openvpn_arg (args, "--cert");
-			add_openvpn_arg (args, cert);
-		}
-
-		if (key && strlen (key)) {
-			add_openvpn_arg (args, "--key");
-			add_openvpn_arg (args, key);
-		}
-	}
-}
-
 static gboolean
 nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
                                  NMSettingVPN *s_vpn,
@@ -824,42 +778,6 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		add_openvpn_arg (args, "nm-openvpn");
 	}
 
-	/* TUN MTU size */
-	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TUNNEL_MTU);
-	if (tmp && strlen (tmp)) {
-		add_openvpn_arg (args, "--tun-mtu");
-		if (!add_openvpn_arg_int (args, tmp)) {
-			g_set_error (error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             "Invalid TUN MTU size '%s'.",
-			             tmp);
-			free_openvpn_args (args);
-			return FALSE;
-		}
-	}
-
-	/* fragment size */
-	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FRAGMENT_SIZE);
-	if (tmp && strlen (tmp)) {
-		add_openvpn_arg (args, "--fragment");
-		if (!add_openvpn_arg_int (args, tmp)) {
-			g_set_error (error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             "Invalid fragment size '%s'.",
-			             tmp);
-			free_openvpn_args (args);
-			return FALSE;
-		}
-	}
-
-	/* mssfix */
-	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_MSSFIX);
-	if (tmp && !strcmp (tmp, "yes")) {
-		add_openvpn_arg (args, "--mssfix");
-	}
-
 	/* Punch script security in the face; this option was added to OpenVPN 2.1-rc9
 	 * and defaults to disallowing any scripts, a behavior change from previous
 	 * versions.
@@ -869,10 +787,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 
 	/* Up script, called when connection has been established or has been restarted */
 	add_openvpn_arg (args, "--up");
-	if (getenv ("OPENVPN_DEBUG"))
-		add_openvpn_arg (args, NM_OPENVPN_HELPER_PATH " --helper-debug");
-	else
-		add_openvpn_arg (args, NM_OPENVPN_HELPER_PATH);
+	add_openvpn_arg (args, NM_OPENVPN_HELPER_PATH);
 	add_openvpn_arg (args, "--up-restart");
 
 	/* Keep key and tun if restart is needed */
@@ -893,7 +808,24 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	/* Now append configuration options which are dependent on the configuration type */
 	if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_TLS)) {
 		add_openvpn_arg (args, "--client");
-		add_cert_args (args, s_vpn);
+
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
+		if (tmp && strlen (tmp)) {
+			add_openvpn_arg (args, "--ca");
+			add_openvpn_arg (args, tmp);
+		}
+
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
+		if (tmp && strlen (tmp)) {
+			add_openvpn_arg (args, "--cert");
+			add_openvpn_arg (args, tmp);
+		}
+
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+		if (tmp && strlen (tmp)) {
+			add_openvpn_arg (args, "--key");
+			add_openvpn_arg (args, tmp);
+		}
 	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY);
 		if (tmp && strlen (tmp)) {
@@ -945,7 +877,25 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		}
 	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 		add_openvpn_arg (args, "--client");
-		add_cert_args (args, s_vpn);
+
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
+		if (tmp && strlen (tmp)) {
+			add_openvpn_arg (args, "--ca");
+			add_openvpn_arg (args, tmp);
+		}
+
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
+		if (tmp && strlen (tmp)) {
+			add_openvpn_arg (args, "--cert");
+			add_openvpn_arg (args, tmp);
+		}
+
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+		if (tmp && strlen (tmp)) {
+			add_openvpn_arg (args, "--key");
+			add_openvpn_arg (args, tmp);
+		}
+
 		/* Use user/path authentication */
 		add_openvpn_arg (args, "--auth-user-pass");
 	} else {
@@ -1007,45 +957,6 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	return TRUE;
 }
 
-static const char *
-check_need_secrets (NMSettingVPN *s_vpn, gboolean *need_secrets)
-{
-	const char *tmp, *key, *ctype;
-
-	g_return_val_if_fail (s_vpn != NULL, FALSE);
-	g_return_val_if_fail (need_secrets != NULL, FALSE);
-
-	*need_secrets = FALSE;
-
-	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
-	ctype = validate_connection_type (tmp);
-	if (!ctype)
-		return NULL;
-
-	if (!strcmp (ctype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		/* Will require a password and maybe private key password */
-		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		if (is_encrypted (key) && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
-			*need_secrets = TRUE;
-
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
-			*need_secrets = TRUE;
-	} else if (!strcmp (ctype, NM_OPENVPN_CONTYPE_PASSWORD)) {
-		/* Will require a password */
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
-			*need_secrets = TRUE;
-	} else if (!strcmp (ctype, NM_OPENVPN_CONTYPE_TLS)) {
-		/* May require private key password */
-		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		if (is_encrypted (key) && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
-			*need_secrets = TRUE;
-	} else {
-		/* Static key doesn't need passwords */
-	}
-
-	return ctype;
-}
-
 static gboolean
 real_connect (NMVPNPlugin   *plugin,
               NMConnection  *connection,
@@ -1054,7 +965,7 @@ real_connect (NMVPNPlugin   *plugin,
 	NMSettingVPN *s_vpn;
 	const char *connection_type;
 	const char *user_name;
-	gboolean need_secrets;
+	const char *tmp;
 
 	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
 	if (!s_vpn) {
@@ -1066,8 +977,11 @@ real_connect (NMVPNPlugin   *plugin,
 		return FALSE;
 	}
 
-	/* Check if we need secrets and validate the connection type */
-	connection_type = check_need_secrets (s_vpn, &need_secrets);
+	user_name = nm_setting_vpn_get_user_name (s_vpn);
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
+	
+	connection_type = validate_connection_type (tmp);
+	
 	if (!connection_type) {
 		g_set_error (error,
 		             NM_VPN_PLUGIN_ERROR,
@@ -1075,9 +989,7 @@ real_connect (NMVPNPlugin   *plugin,
 		             "%s",
 		             "Invalid connection type.");
 		return FALSE;
-	}
-
-	user_name = nm_setting_vpn_get_user_name (s_vpn);
+	}	
 
 	/* Need a username for any password-based connection types */
 	if (   !strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)
@@ -1096,8 +1008,8 @@ real_connect (NMVPNPlugin   *plugin,
 	if (!nm_openvpn_properties_validate (s_vpn, error))
 		return FALSE;
 
-	/* Validate secrets */
-	if (need_secrets) {
+	/* Static Key doesn't need secrets; the rest do */
+	if (strcmp (connection_type, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		if (!nm_openvpn_secrets_validate (s_vpn, error))
 			return FALSE;
 	}
@@ -1118,6 +1030,7 @@ real_need_secrets (NMVPNPlugin *plugin,
 	NMSettingVPN *s_vpn;
 	const char *connection_type;
 	gboolean need_secrets = FALSE;
+	const char *tmp;
 
 	g_return_val_if_fail (NM_IS_VPN_PLUGIN (plugin), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
@@ -1132,7 +1045,9 @@ real_need_secrets (NMVPNPlugin *plugin,
 		return FALSE;
 	}
 
-	connection_type = check_need_secrets (s_vpn, &need_secrets);
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
+	connection_type = validate_connection_type (tmp);
+	
 	if (!connection_type) {
 		g_set_error (error,
 		             NM_VPN_PLUGIN_ERROR,
@@ -1142,6 +1057,23 @@ real_need_secrets (NMVPNPlugin *plugin,
 		return FALSE;
 	}
 
+	if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
+		/* Will require a password and maybe private key password */
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
+			need_secrets = TRUE;
+
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
+			need_secrets = TRUE;
+	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD)) {
+		/* Will require a password */
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
+			need_secrets = TRUE;
+	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_TLS)) {
+		/* May require private key password */
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
+			need_secrets = TRUE;
+	}
+
 	if (need_secrets)
 		*setting_name = NM_SETTING_VPN_SETTING_NAME;
 
